@@ -25,8 +25,8 @@
 
 #include "hitag.h"
 
-void SimulateHitagSTag(bool tag_mem_supplied, uint8_t *data, bool ledcontrol);
-void ReadHitagS(hitag_function htf, hitag_data *htd, bool ledcontrol);
-void WritePageHitagS(hitag_function htf, hitag_data *htd, int page, bool ledcontrol);
-void Hitag_check_challenges(uint8_t *data, uint32_t datalen, bool ledcontrol);
+void SimulateHitagSTag(bool tag_mem_supplied, const uint8_t *data, bool ledcontrol);
+void ReadHitagS(hitag_function htf, const hitag_data *htd, bool ledcontrol);
+void WritePageHitagS(hitag_function htf, const hitag_data *htd, int page, bool ledcontrol);
+void Hitag_check_challenges(const uint8_t *data, uint32_t datalen, bool ledcontrol);
 #endif
