@@ -25,7 +25,7 @@
 void LegicRfInfo(void);
 int LegicRfReaderEx(uint16_t offset, uint16_t len, uint8_t iv);
 void LegicRfReader(uint16_t offset, uint16_t len, uint8_t iv);
-void LegicRfWriter(uint16_t offset, uint16_t len, uint8_t iv, uint8_t *data);
+void LegicRfWriter(uint16_t offset, uint16_t len, uint8_t iv, const uint8_t *data);
 
 legic_card_select_t *getLegicCardInfo(void);
 #endif /* __LEGICRF_H */
