@@ -235,14 +235,14 @@ static int CmdHFFudanReader(const char *Cmd) {
 static int CmdHFFudanDump(const char *Cmd) {
     CLIParserContext *ctx;
     CLIParserInit(&ctx, "hf fudan dump",
-                  "Dump FUDAN tag to binary file\n"
+                  "Dump FUDAN tag to file (bin/json)\n"
                   "If no <name> given, UID will be used as filename",
                   "hf fudan dump -f mydump        --> dump using filename\n"
                  );
 
     void *argtable[] = {
         arg_param_begin,
-        arg_str0("f", "file", "<fn>", "filename of dump"),
+        arg_str0("f", "file", "<fn>", "Specify a filename for dump file"),
         arg_param_end
     };
     CLIExecWithReturn(ctx, Cmd, argtable, true);
@@ -339,14 +339,7 @@ static int CmdHFFudanDump(const char *Cmd) {
         free(fptr);
     }
 
-    saveFile(dataFilename, ".bin", (uint8_t *)carddata, sizeof(carddata));
-    saveFileEML(dataFilename, (uint8_t *)carddata, sizeof(carddata), MAX_FUDAN_BLOCK_SIZE);
-
-    iso14a_mf_extdump_t xdump;
-    xdump.card_info = card;
-    xdump.dump = (uint8_t *)carddata;
-    xdump.dumplen = sizeof(carddata);
-    saveFileJSON(dataFilename, jsfFudan, (uint8_t *)&xdump, sizeof(xdump), NULL);
+    pm3_save_dump(dataFilename, (uint8_t *)carddata, sizeof(carddata), jsfFudan);
     return PM3_SUCCESS;
 }
 
@@ -460,7 +453,7 @@ static int CmdHFFudanView(const char *Cmd) {
                  );
     void *argtable[] = {
         arg_param_begin,
-        arg_str1("f", "file", "<fn>", "filename of dump"),
+        arg_str1("f", "file", "<fn>", "Specify a filename for dump file"),
         arg_param_end
     };
     CLIExecWithReturn(ctx, Cmd, argtable, false);
