@@ -22,6 +22,6 @@
 
 int CmdLFParadox(const char *Cmd);
 
-int demodParadox(bool verbose);
+int demodParadox(bool verbose, bool oldChksum);
 int detectParadox(uint8_t *dest, size_t *size, int *wave_start_idx);
 #endif
