@@ -71,6 +71,7 @@ static const AllowedChannelModes_t AllowedChannelModes[] = {
     {MFDES_CREATE_VALUE_FILE,         DACd40,  DCCNative,    DCMMACed},
     {MFDES_CREATE_LINEAR_RECORD_FILE, DACd40,  DCCNative,    DCMMACed},
     {MFDES_CREATE_CYCLIC_RECORD_FILE, DACd40,  DCCNative,    DCMMACed},
+    {MFDES_DELETE_FILE,               DACd40,  DCCNative,    DCMMACed},
     {MFDES_COMMIT_TRANSACTION,        DACd40,  DCCNative,    DCMMACed},
     {MFDES_CLEAR_RECORD_FILE,         DACd40,  DCCNative,    DCMMACed},
     {MFDES_GET_FILE_SETTINGS,         DACd40,  DCCNative,    DCMMACed},
@@ -118,6 +119,7 @@ static const AllowedChannelModes_t AllowedChannelModes[] = {
     {MFDES_CREATE_VALUE_FILE,         DACEV1,  DCCNative,    DCMMACed},
     {MFDES_CREATE_LINEAR_RECORD_FILE, DACEV1,  DCCNative,    DCMMACed},
     {MFDES_CREATE_CYCLIC_RECORD_FILE, DACEV1,  DCCNative,    DCMMACed},
+    {MFDES_DELETE_FILE,               DACEV1,  DCCNative,    DCMMACed},
     {MFDES_GET_VALUE,                 DACEV1,  DCCNative,    DCMMACed},
     {MFDES_CREDIT,                    DACEV1,  DCCNative,    DCMMACed},
     {MFDES_LIMITED_CREDIT,            DACEV1,  DCCNative,    DCMMACed},
