@@ -42,8 +42,9 @@ const static vocabulary_t vocabulary[] = {
     { 1, "prefs help" },
     { 1, "prefs show" },
     { 1, "prefs get barmode" },
-    { 1, "prefs get clientdebug" },
-    { 1, "prefs get clientdelay" },
+    { 1, "prefs get client.debug" },
+    { 1, "prefs get client.delay" },
+    { 1, "prefs get client.timeout" },
     { 1, "prefs get color" },
     { 1, "prefs get savepaths" },
     { 1, "prefs get emoji" },
@@ -52,8 +53,9 @@ const static vocabulary_t vocabulary[] = {
     { 1, "prefs get plotsliders" },
     { 1, "prefs set help" },
     { 1, "prefs set barmode" },
-    { 1, "prefs set clientdebug" },
-    { 1, "prefs set clientdelay" },
+    { 1, "prefs set client.debug" },
+    { 1, "prefs set client.delay" },
+    { 1, "prefs set client.timeout" },
     { 1, "prefs set color" },
     { 1, "prefs set emoji" },
     { 1, "prefs set hints" },
@@ -105,6 +107,7 @@ const static vocabulary_t vocabulary[] = {
     { 1, "data atr" },
     { 1, "data bin2hex" },
     { 0, "data bitsamples" },
+    { 1, "data bmap" },
     { 1, "data clear" },
     { 1, "data diff" },
     { 0, "data hexsamples" },
@@ -269,13 +272,14 @@ const static vocabulary_t vocabulary[] = {
     { 1, "hf iclass help" },
     { 1, "hf iclass list" },
     { 0, "hf iclass dump" },
-    { 1, "hf iclass info" },
+    { 0, "hf iclass info" },
     { 0, "hf iclass rdbl" },
     { 0, "hf iclass reader" },
     { 0, "hf iclass restore" },
     { 0, "hf iclass sniff" },
     { 1, "hf iclass view" },
     { 0, "hf iclass wrbl" },
+    { 0, "hf iclass creditepurse" },
     { 0, "hf iclass chk" },
     { 1, "hf iclass loclass" },
     { 1, "hf iclass lookup" },
@@ -284,7 +288,7 @@ const static vocabulary_t vocabulary[] = {
     { 0, "hf iclass esave" },
     { 0, "hf iclass esetblk" },
     { 0, "hf iclass eview" },
-    { 1, "hf iclass configcard" },
+    { 0, "hf iclass configcard" },
     { 1, "hf iclass calcnewkey" },
     { 1, "hf iclass encode" },
     { 1, "hf iclass encrypt" },
@@ -362,17 +366,20 @@ const static vocabulary_t vocabulary[] = {
     { 0, "hf mf gen3uid" },
     { 0, "hf mf gen3blk" },
     { 0, "hf mf gen3freeze" },
+    { 0, "hf mf ginfo" },
     { 0, "hf mf ggetblk" },
     { 0, "hf mf gload" },
     { 0, "hf mf gsave" },
     { 0, "hf mf gsetblk" },
     { 0, "hf mf gview" },
+    { 0, "hf mf gchpwd" },
     { 0, "hf mf gdmcfg" },
     { 0, "hf mf gdmsetcfg" },
     { 0, "hf mf gdmsetblk" },
     { 0, "hf mf ndefformat" },
     { 0, "hf mf ndefread" },
     { 0, "hf mf ndefwrite" },
+    { 0, "hf mf encodehid" },
     { 1, "hf mfp help" },
     { 1, "hf mfp list" },
     { 0, "hf mfp auth" },
@@ -450,8 +457,13 @@ const static vocabulary_t vocabulary[] = {
     { 1, "hf mfdes test" },
     { 1, "hf ntag424 help" },
     { 0, "hf ntag424 info" },
-    { 0, "hf ntag424 sdm" },
     { 1, "hf ntag424 view" },
+    { 0, "hf ntag424 auth" },
+    { 0, "hf ntag424 read" },
+    { 0, "hf ntag424 write" },
+    { 0, "hf ntag424 getfs" },
+    { 0, "hf ntag424 changefs" },
+    { 0, "hf ntag424 changekey" },
     { 1, "hf seos help" },
     { 0, "hf seos info" },
     { 1, "hf seos list" },
@@ -779,6 +791,7 @@ const static vocabulary_t vocabulary[] = {
     { 1, "smart help" },
     { 1, "smart list" },
     { 0, "smart info" },
+    { 0, "smart relay" },
     { 0, "smart reader" },
     { 0, "smart raw" },
     { 1, "smart upgrade" },
