@@ -56,12 +56,32 @@ typedef struct {
     uint8_t ats[256];
 } PACKED iso14a_card_select_t;
 
+typedef struct {
+    uint8_t uid[10];
+    uint8_t uidlen;
+    uint8_t atqa[2];
+    uint8_t sak;
+    uint8_t ats_len;
+    uint8_t ats[256];
+    uint8_t signature[32];
+} PACKED iso14a_card_select_ev1_t;
+
 typedef struct {
     iso14a_card_select_t card_info;
-    uint8_t *dump;
     uint16_t dumplen;
+    uint8_t *dump;
 } iso14a_mf_extdump_t;
 
+typedef struct {
+    union {
+        iso14a_card_select_t mfc;
+        iso14a_card_select_ev1_t ev1;
+    } card;
+    uint16_t dumplen;
+    uint8_t *dump;
+} iso14a_mf_dump_ev1_t;
+
+
 typedef enum ISO14A_COMMAND {
     ISO14A_CONNECT = (1 << 0),
     ISO14A_NO_DISCONNECT = (1 << 1),
